@@ -16,6 +16,7 @@
 //! documentation, and implementation specific examples with each implementation module.
 
 #[cfg(any(feature = "cb-channel-exec", feature = "workstealing-exec"))]
+#[macro_use]
 extern crate crossbeam_channel;
 #[cfg(feature = "workstealing-exec")]
 extern crate crossbeam_deque;
@@ -48,8 +49,4 @@ use synchronoise::CountdownEvent;
 
 #[cfg(test)]
 mod tests {
-    #[test]
-    fn it_works() {
-        assert_eq!(2 + 2, 4);
-    }
 }
